@@ -12,17 +12,18 @@ from axes.models import AccessAttempt
 log = getLogger(__name__)
 
 
-def get_cool_off_threshold(attempt_time: Optional[datetime] = None) -> datetime:
+def get_cool_off_threshold(request: Optional[HttpRequest] = None) -> datetime:
     """
     Get threshold for fetching access attempts from the database.
     """
 
-    cool_off = get_cool_off()
+    cool_off = get_cool_off(request)
     if cool_off is None:
         raise TypeError(
             "Cool off threshold can not be calculated with settings.AXES_COOLOFF_TIME set to None"
         )
 
+    attempt_time = request.axes_attempt_time
     if attempt_time is None:
         return now() - cool_off
     return attempt_time - cool_off
@@ -38,7 +39,7 @@ def filter_user_attempts(
     username = get_client_username(request, credentials)
 
     filter_kwargs_list = get_client_parameters(
-        username, request.axes_ip_address, request.axes_user_agent
+        username, request.axes_ip_address, request.axes_user_agent, request, credentials
     )
     attempts_list = [
         AccessAttempt.objects.filter(**filter_kwargs)
@@ -62,12 +63,12 @@ def get_user_attempts(
         )
         return attempts_list
 
-    threshold = get_cool_off_threshold(request.axes_attempt_time)
+    threshold = get_cool_off_threshold(request)
     log.debug("AXES: Getting access attempts that are newer than %s", threshold)
     return [attempts.filter(attempt_time__gte=threshold) for attempts in attempts_list]
 
 
-def clean_expired_user_attempts(attempt_time: Optional[datetime] = None) -> int:
+def clean_expired_user_attempts(request: Optional[HttpRequest] = None) -> int:
     """
     Clean expired user attempts from the database.
     """
@@ -78,7 +79,7 @@ def clean_expired_user_attempts(attempt_time: Optional[datetime] = None) -> int:
         )
         return 0
 
-    threshold = get_cool_off_threshold(attempt_time)
+    threshold = get_cool_off_threshold(request)
     count, _ = AccessAttempt.objects.filter(attempt_time__lt=threshold).delete()
     log.info(
         "AXES: Cleaned up %s expired access attempts from database that were older than %s",
