@@ -6,7 +6,7 @@ from axes.utils import reset
 class Command(BaseCommand):
     help = "Reset all access attempts and lockouts"
 
-    def handle(self, *args, **options):  # pylint: disable=unused-argument
+    def handle(self, *args, **options):
         count = reset()
 
         if count:
